#include "stdinc.h"
#include "EscWeb.h"

#define CYCLE_MAX 500

bool gbEscPower;
bool gbEscLamp;
BYTE gnEscTimerMin, gnEscTimerSec, gnEscPower=100;
bool gbEscRun;
static WORD nCycleCounter;
static WORD nCycleActive;
static DWORD dwCycleTick;
static bool bTestButton;
static bool bDoorOpen;
static DWORD dwToneTick;

unsigned char ReadRTC(unsigned char nAddr);

static void SetOutput(BYTE nMask, bool bOn)
{	if (bOn)
	{	PortBits(PORT_IOLOW, nMask, false);
		PortBits(PORT_IOHIGH, nMask, false);
	} else
	{	PortBits(PORT_IOHIGH, nMask, true);
		PortBits(PORT_IOLOW, nMask, true);
	}
}

void EscPower(bool bOn)
{	dprintf(PRINTF_DEBUG, "EscPower(%u)\n", bOn);
	SetOutput(1, bOn);
	gbEscPower = bOn;
}

void EscLamp(bool bOn)
{	dprintf(PRINTF_DEBUG, "EscLamp(%u)\n", bOn);
	SetOutput(2, bOn);
	gbEscLamp = bOn;
}

void EscStartRun(BYTE nMin, BYTE nSec, BYTE nPower)
{	SysBeginCritical();
	gnEscTimerMin = nMin;
	gnEscTimerSec = nSec;
	gnEscPower = nPower;
	nCycleCounter = 0;
	nCycleActive = (WORD)(gnEscPower * 5);
	SetOutput(3, true);		//	turn on both power and lamp
	dwCycleTick = nTickCount;
	gbEscRun = true;
	SysEndCritical();
}

void EscStopRun(void)
{	SysBeginCritical();
	gbEscRun = false;
	EscPower(bTestButton);
	EscLamp(bDoorOpen);
	SysEndCritical();
}

void EscAjaxString(char * pBuff)
{	SysBeginCritical();
	if (gbEscRun)
	{	if (gnEscTimerMin)
			pBuff += sprintf(pBuff, "%2u%02u", gnEscTimerMin, gnEscTimerSec);
		else
			pBuff += sprintf(pBuff, "%4u", gnEscTimerSec);
	} else
	{	BYTE h, m;
		h = ReadRTC(4);
		m = ReadRTC(2);
		if (h > 12)
			h -= 12;
		else if (! h)
			h = 12;
		pBuff += sprintf(pBuff, "%2u%02u", h, m);
	}
	pBuff += sprintf(pBuff, ",%u,%u,%u", gnEscPower, gbEscRun, bDoorOpen);
	SysEndCritical();
}

void EscSupportProc(void)
{	DWORD t, tNow;
	bool bTest, bDoor;
	BYTE nInput;
	SetOutput(3, false);
	gbEscRun = false;
	while (1)
	{	SysYield();
		nInput = *((BYTE *)0x00600070);
		bTest = (bool)((nInput & 0x80) == 0);
		bDoor = (bool)((nInput & 0x40) != 0);
		if (bTest != bTestButton)
		{	bTestButton = bTest;
			dprintf(PRINTF_DEBUG, "Test button: %u\n", bTest);
			EscPower(bTest);
		}
		if (bDoor != bDoorOpen)
		{	bDoorOpen = bDoor;
			dprintf(PRINTF_DEBUG, "Door switch: %u\n", bDoor);
			if (bDoor)
				EscStopRun();
			else
				EscLamp(false);
		}
		if (dwToneTick && (nTickCount - dwToneTick) >= 100)
		{	dwToneTick = 0;
			SpeakerTone(0);
		}
		SysBeginCritical();
		if (gbEscRun)
		{	tNow = nTickCount;
			t = tNow - dwCycleTick;
			dwCycleTick = tNow;
			while (t)
			{	t--;
				nCycleCounter++;
				if (nCycleCounter == CYCLE_MAX)
				{	nCycleCounter = 0;
					EscPower(true);
				} else if (nCycleCounter == nCycleActive)
					EscPower(bTestButton);
				if ((nCycleCounter % 100) == 0)
				{	if (gnEscTimerSec)
						gnEscTimerSec--;
					else
					{	gnEscTimerMin--;
						gnEscTimerSec = 59;
					}
					if (gnEscTimerMin == 0 && gnEscTimerSec == 0)
					{	gbEscRun = false;
						EscPower(bTestButton);
						EscLamp(bDoorOpen);
						SpeakerTone(4);
						dwToneTick = nTickCount;
						break;
					}
				}
			}
		}
		SysEndCritical();
	}
}
